@@ -2,7 +2,7 @@
  *
  * pqxx::connection encapsulates a connection to a database.
  *
- * Copyright (c) 2000-2023, Jeroen T. Vermeulen.
+ * Copyright (c) 2000-2024, Jeroen T. Vermeulen.
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this
@@ -93,7 +93,7 @@ void PQXX_COLD PQXX_LIBEXPORT pqxx::internal::skip_init_ssl(int skips) noexcept
 std::string PQXX_COLD
 pqxx::encrypt_password(char const user[], char const password[])
 {
-  std::unique_ptr<char, void (*)(void const *)> p{
+  std::unique_ptr<char, void (*)(void const *)> const p{
     PQencryptPassword(password, user), pqxx::internal::pq::pqfreemem};
   return {p.get()};
 }
@@ -114,7 +114,11 @@ pqxx::connection::connection(
   if (m_conn == nullptr)
     throw std::bad_alloc{};
   if (status() == CONNECTION_BAD)
+  {
+    PQfinish(m_conn);
+    m_conn = nullptr;
     throw pqxx::broken_connection{PQerrorMessage(m_conn)};
+  }
 }
 
 
@@ -219,32 +223,23 @@ pqxx::result pqxx::connection::make_result(
   internal::pq::PGresult *pgr, std::shared_ptr<std::string> const &query,
   std::string_view desc)
 {
-  if (pgr == nullptr)
+  std::shared_ptr<internal::pq::PGresult> const smart{
+    pgr, internal::clear_result};
+  if (not smart)
   {
     if (is_open())
       throw failure(err_msg());
     else
       throw broken_connection{"Lost connection to the database server."};
   }
-  internal::encoding_group enc;
-  try
-  {
-    enc = internal::enc_group(encoding_id());
-  }
-  catch (std::exception const &)
-  {
-    // Don't let the PGresult leak.
-    // TODO: Can we just accept a unique_ptr instead?
-    internal::clear_result(pgr);
-    throw;
-  }
-  auto const r{pqxx::internal::gate::result_creation::create(pgr, query, enc)};
+  auto const enc{internal::enc_group(encoding_id())};
+  auto r{pqxx::internal::gate::result_creation::create(smart, query, enc)};
   pqxx::internal::gate::result_creation{r}.check_status(desc);
   return r;
 }
 
 
-int PQXX_COLD pqxx::connection::backendpid() const &noexcept
+int PQXX_COLD pqxx::connection::backendpid() const & noexcept
 {
   return (m_conn == nullptr) ? 0 : PQbackendPID(m_conn);
 }
@@ -259,7 +254,7 @@ PQXX_PURE int socket_of(::pqxx::internal::pq::PGconn const *c) noexcept
 } // namespace
 
 
-int pqxx::connection::sock() const &noexcept
+int pqxx::connection::sock() const & noexcept
 {
   return socket_of(m_conn);
 }
@@ -316,7 +311,8 @@ void pqxx::connection::set_up_state()
         "Unsupported frontend/backend protocol version; 3.0 is the minimum."};
   }
 
-  if (server_version() <= 90000)
+  constexpr int oldest_server{90000};
+  if (server_version() <= oldest_server)
     throw feature_not_supported{
       "Unsupported server version; 9.0 is the minimum."};
 
@@ -344,8 +340,7 @@ void pqxx::connection::process_notice_raw(char const msg[]) noexcept
     return;
   auto const rbegin = std::crbegin(m_errorhandlers),
              rend = std::crend(m_errorhandlers);
-  for (auto i{rbegin}; (i != rend) and (**i)(msg); ++i)
-    ;
+  for (auto i{rbegin}; (i != rend) and (**i)(msg); ++i);
 }
 
 
@@ -354,9 +349,10 @@ void pqxx::connection::process_notice(char const msg[]) noexcept
   if (msg == nullptr)
     return;
   zview const view{msg};
+  // TODO: A multibyte message could end in a '\n' that's not a newline!
   if (std::empty(view))
     return;
-  else if (msg[std::size(view) - 1] == '\n')
+  else if (view.back() == '\n')
     process_notice_raw(msg);
   else
     // Newline is missing.  Let the zview version of the code add it.
@@ -452,7 +448,7 @@ pqxx::connection::remove_receiver(pqxx::notification_receiver *T) noexcept
       bool const gone{R.second == ++R.first};
       m_receivers.erase(i);
       if (gone)
-        exec(internal::concat("UNLISTEN ", quote_name(needle.first)).c_str());
+        exec(internal::concat("UNLISTEN ", quote_name(needle.first)));
     }
   }
   catch (std::exception const &e)
@@ -482,21 +478,24 @@ void wrap_pgfreecancel(PGcancel *ptr)
 {
   PQfreeCancel(ptr);
 }
+
+
+/// A fairly arbitrary buffer size for error strings and such.
+constexpr int buf_size{500u};
 } // namespace
 
 
 void PQXX_COLD pqxx::connection::cancel_query()
 {
-  std::unique_ptr<PGcancel, void (*)(PGcancel *)> cancel{
+  std::unique_ptr<PGcancel, void (*)(PGcancel *)> const cancel{
     PQgetCancel(m_conn), wrap_pgfreecancel};
   if (cancel == nullptr)
     PQXX_UNLIKELY
   throw std::bad_alloc{};
 
-  std::array<char, 500u> errbuf;
+  std::array<char, buf_size> errbuf{};
   auto const err{errbuf.data()};
-  auto const c{
-    PQcancel(cancel.get(), err, static_cast<int>(std::size(errbuf)))};
+  auto const c{PQcancel(cancel.get(), err, buf_size)};
   if (c == 0)
     PQXX_UNLIKELY
   throw pqxx::sql_error{std::string{err, std::size(errbuf)}, "[cancel]"};
@@ -511,13 +510,13 @@ void pqxx::connection::set_blocking(bool block) &
   unsigned long mode{not block};
   if (::ioctlsocket(fd, FIONBIO, &mode) != 0)
   {
-    std::array<char, 200> errbuf;
+    std::array<char, buf_size> errbuf{};
     char const *err{pqxx::internal::error_string(WSAGetLastError(), errbuf)};
     throw broken_connection{
       internal::concat("Could not set socket's blocking mode: ", err)};
   }
 #  else  // _WIN32
-  std::array<char, 200> errbuf;
+  std::array<char, buf_size> errbuf{};
   auto flags{::fcntl(fd, F_GETFL, 0)};
   if (flags == -1)
   {
@@ -541,7 +540,7 @@ void pqxx::connection::set_blocking(bool block) &
 
 
 void PQXX_COLD
-pqxx::connection::set_verbosity(error_verbosity verbosity) &noexcept
+pqxx::connection::set_verbosity(error_verbosity verbosity) & noexcept
 {
   PQsetErrorVerbosity(m_conn, static_cast<PGVerbosity>(verbosity));
 }
@@ -556,7 +555,7 @@ using notify_ptr = std::unique_ptr<PGnotify, void (*)(void const *)>;
 /// Get one notification from a connection, or null.
 notify_ptr get_notif(pqxx::internal::pq::PGconn *conn)
 {
-  return notify_ptr(PQnotifies(conn), pqxx::internal::pq::pqfreemem);
+  return {PQnotifies(conn), pqxx::internal::pq::pqfreemem};
 }
 } // namespace
 
@@ -580,7 +579,7 @@ int pqxx::connection::get_notifs()
     auto const Hit{m_receivers.equal_range(std::string{N->relname})};
     if (Hit.second != Hit.first)
     {
-      std::string payload{N->extra};
+      std::string const payload{N->extra};
       for (auto i{Hit.first}; i != Hit.second; ++i) try
         {
           (*i->second)(payload, N->be_pid);
@@ -689,9 +688,9 @@ pqxx::connection::exec(std::string_view query, std::string_view desc)
 
 
 pqxx::result pqxx::connection::exec(
-  std::shared_ptr<std::string> query, std::string_view desc)
+  std::shared_ptr<std::string> const &query, std::string_view desc)
 {
-  auto const res{make_result(PQexec(m_conn, query->c_str()), query, desc)};
+  auto res{make_result(PQexec(m_conn, query->c_str()), query, desc)};
   get_notifs();
   return res;
 }
@@ -701,9 +700,9 @@ std::string pqxx::connection::encrypt_password(
   char const user[], char const password[], char const *algorithm)
 {
   auto const buf{PQencryptPasswordConn(m_conn, password, user, algorithm)};
-  std::unique_ptr<char const, void (*)(void const *)> ptr{
+  std::unique_ptr<char const, void (*)(void const *)> const ptr{
     buf, pqxx::internal::pq::pqfreemem};
-  return std::string(ptr.get());
+  return (ptr.get());
 }
 
 
@@ -739,7 +738,7 @@ pqxx::result pqxx::connection::exec_prepared(
     args.values.data(), args.lengths.data(),
     reinterpret_cast<int const *>(args.formats.data()),
     static_cast<int>(format::text))};
-  auto const r{make_result(pq_result, q, statement)};
+  auto r{make_result(pq_result, q, statement)};
   get_notifs();
   return r;
 }
@@ -747,6 +746,9 @@ pqxx::result pqxx::connection::exec_prepared(
 
 void pqxx::connection::close()
 {
+  // Just in case PQfinish() doesn't handle nullptr nicely.
+  if (m_conn == nullptr)
+    return;
   try
   {
     if (m_trans)
@@ -876,7 +878,7 @@ void pqxx::connection::write_copy_line(std::string_view line)
 
 void pqxx::connection::end_copy_write()
 {
-  int res{PQputCopyEnd(m_conn, nullptr)};
+  int const res{PQputCopyEnd(m_conn, nullptr)};
   switch (res)
   {
   case -1:
@@ -939,8 +941,7 @@ pqxx::connection::esc_raw(unsigned char const bin[], std::size_t len) const
 }
 
 
-std::string
-pqxx::connection::esc_raw(std::basic_string_view<std::byte> bin) const
+std::string pqxx::connection::esc_raw(bytes_view bin) const
 {
   return pqxx::internal::esc_bin(bin);
 }
@@ -961,9 +962,8 @@ std::string PQXX_COLD pqxx::connection::unesc_raw(char const text[]) const
   {
     // Legacy escape format.
     // TODO: Remove legacy support.
-    std::size_t len;
-    auto bytes{const_cast<unsigned char *>(
-      reinterpret_cast<unsigned char const *>(text))};
+    std::size_t len{};
+    auto bytes{reinterpret_cast<unsigned char const *>(text)};
     std::unique_ptr<unsigned char, void (*)(void const *)> const ptr{
       PQunescapeBytea(bytes, &len), pqxx::internal::pq::pqfreemem};
     return std::string{ptr.get(), ptr.get() + len};
@@ -978,8 +978,7 @@ pqxx::connection::quote_raw(unsigned char const bin[], std::size_t len) const
 }
 
 
-std::string
-pqxx::connection::quote_raw(std::basic_string_view<std::byte> bytes) const
+std::string pqxx::connection::quote_raw(bytes_view bytes) const
 {
   return internal::concat("'", esc_raw(bytes), "'::bytea");
 }
@@ -991,7 +990,7 @@ std::string PQXX_COLD pqxx::connection::quote(binarystring const &b) const
 }
 
 
-std::string pqxx::connection::quote(std::basic_string_view<std::byte> b) const
+std::string pqxx::connection::quote(bytes_view b) const
 {
   return internal::concat("'", esc_raw(b), "'::bytea");
 }
@@ -999,10 +998,10 @@ std::string pqxx::connection::quote(std::basic_string_view<std::byte> b) const
 
 std::string pqxx::connection::quote_name(std::string_view identifier) const
 {
-  std::unique_ptr<char, void (*)(void const *)> buf{
+  std::unique_ptr<char, void (*)(void const *)> const buf{
     PQescapeIdentifier(m_conn, identifier.data(), std::size(identifier)),
     pqxx::internal::pq::pqfreemem};
-  if (buf.get() == nullptr)
+  if (buf == nullptr)
     PQXX_UNLIKELY
   throw failure{err_msg()};
   return std::string{buf.get()};
@@ -1058,7 +1057,7 @@ int pqxx::connection::await_notification()
 int pqxx::connection::await_notification(
   std::time_t seconds, long microseconds)
 {
-  int notifs = get_notifs();
+  int const notifs = get_notifs();
   if (notifs == 0)
   {
     PQXX_LIKELY
@@ -1141,7 +1140,7 @@ pqxx::result pqxx::connection::exec_params(
     args.values.data(), args.lengths.data(),
     reinterpret_cast<int const *>(args.formats.data()),
     static_cast<int>(format::text))};
-  auto const r{make_result(pq_result, q)};
+  auto r{make_result(pq_result, q)};
   get_notifs();
   return r;
 }
@@ -1195,7 +1194,7 @@ std::string pqxx::connection::connection_string() const
 
   std::unique_ptr<PQconninfoOption, void (*)(PQconninfoOption *)> const params{
     PQconninfo(m_conn), pqconninfofree};
-  if (params.get() == nullptr)
+  if (params == nullptr)
     PQXX_UNLIKELY
   throw std::bad_alloc{};
 
