@@ -2,7 +2,7 @@
  *
  * Allows direct access to large objects, as well as though I/O streams.
  *
- * Copyright (c) 2000-2023, Jeroen T. Vermeulen.
+ * Copyright (c) 2000-2024, Jeroen T. Vermeulen.
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this
@@ -67,10 +67,10 @@ constexpr int PQXX_COLD std_dir_to_pq_dir(std::ios::seekdir dir) noexcept
 } // namespace
 
 
-PQXX_COLD pqxx::largeobject::largeobject(dbtransaction &t)
+PQXX_COLD pqxx::largeobject::largeobject(dbtransaction &t) :
+        m_id{lo_creat(raw_connection(t), 0)}
 {
   // (Mode is ignored as of postgres 8.1.)
-  m_id = lo_creat(raw_connection(t), 0);
   if (m_id == oid_none)
   {
     int const err{errno};
@@ -83,9 +83,9 @@ PQXX_COLD pqxx::largeobject::largeobject(dbtransaction &t)
 
 
 PQXX_COLD
-pqxx::largeobject::largeobject(dbtransaction &t, std::string_view file)
+pqxx::largeobject::largeobject(dbtransaction &t, std::string_view file) :
+        m_id{lo_import(raw_connection(t), std::data(file))}
 {
-  m_id = lo_import(raw_connection(t), std::data(file));
   if (m_id == oid_none)
   {
     int const err{errno};
