@@ -2,7 +2,7 @@
  *
  * pqxx::result represents the set of result rows from a database query
  *
- * Copyright (c) 2000-2023, Jeroen T. Vermeulen.
+ * Copyright (c) 2000-2024, Jeroen T. Vermeulen.
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this
@@ -33,7 +33,7 @@ extern "C"
 namespace pqxx
 {
 PQXX_DECLARE_ENUM_CONVERSION(ExecStatusType);
-}
+} // namespace pqxx
 
 std::string const pqxx::result::s_empty_string;
 
@@ -46,9 +46,9 @@ void pqxx::internal::clear_result(pq::PGresult const *data) noexcept
 
 
 pqxx::result::result(
-  pqxx::internal::pq::PGresult *rhs, std::shared_ptr<std::string> query,
-  internal::encoding_group enc) :
-        m_data{make_data_pointer(rhs)}, m_query{query}, m_encoding(enc)
+  std::shared_ptr<pqxx::internal::pq::PGresult> const &rhs,
+  std::shared_ptr<std::string> const &query, internal::encoding_group enc) :
+        m_data{rhs}, m_query{query}, m_encoding(enc)
 {}
 
 
@@ -141,13 +141,13 @@ pqxx::row pqxx::result::operator[](result_size_type i) const noexcept
 }
 
 
-#if pqxx_have_multidim
+#if defined(PQXX_HAVE_MULTIDIM)
 pqxx::field pqxx::result::operator[](
   result_size_type row_num, row_size_type col_num) const noexcept
 {
   return {*this, row_num, col_num};
 }
-#endif // pqxx_have_multidim
+#endif // PQXX_HAVE_MULTIDIM
 
 
 pqxx::row pqxx::result::at(pqxx::result::size_type i) const
@@ -319,11 +319,9 @@ std::string pqxx::result::status_error() const
   case PGRES_EMPTY_QUERY: // The string sent to the backend was empty.
   case PGRES_COMMAND_OK:  // Successful completion, no result data.
   case PGRES_TUPLES_OK:   // The query successfully executed.
-    break;
-
-  case PGRES_COPY_OUT:  // Copy Out (from server) data transfer started.
-  case PGRES_COPY_IN:   // Copy In (to server) data transfer started.
-  case PGRES_COPY_BOTH: // Copy In/Out.  Used for streaming replication.
+  case PGRES_COPY_OUT:    // Copy Out (from server) data transfer started.
+  case PGRES_COPY_IN:     // Copy In (to server) data transfer started.
+  case PGRES_COPY_BOTH:   // Copy In/Out.  Used for streaming replication.
     break;
 
 #if defined(LIBPQ_HAS_PIPELINING)
@@ -357,9 +355,9 @@ char const *pqxx::result::cmd_status() const noexcept
 }
 
 
-std::string const &pqxx::result::query() const &noexcept
+std::string const &pqxx::result::query() const & noexcept
 {
-  return (m_query.get() == nullptr) ? s_empty_string : *m_query;
+  return (m_query == nullptr) ? s_empty_string : *m_query;
 }
 
 
@@ -368,7 +366,7 @@ pqxx::oid pqxx::result::inserted_oid() const
   if (m_data.get() == nullptr)
     throw usage_error{
       "Attempt to read oid of inserted row without an INSERT result"};
-  return PQoidValue(const_cast<internal::pq::PGresult *>(m_data.get()));
+  return PQoidValue(m_data.get());
 }
 
 
@@ -414,8 +412,7 @@ pqxx::oid pqxx::result::column_type(row::size_type col_num) const
 
 pqxx::row::size_type pqxx::result::column_number(zview col_name) const
 {
-  auto const n{PQfnumber(
-    const_cast<internal::pq::PGresult *>(m_data.get()), col_name.c_str())};
+  auto const n{PQfnumber(m_data.get(), col_name.c_str())};
   if (n == -1)
     throw argument_error{
       internal::concat("Unknown column name: '", col_name, "'.")};
@@ -469,9 +466,7 @@ int pqxx::result::errorposition() const
   int pos{-1};
   if (m_data.get())
   {
-    auto const p{PQresultErrorField(
-      const_cast<internal::pq::PGresult *>(m_data.get()),
-      PG_DIAG_STATEMENT_POSITION)};
+    auto const p{PQresultErrorField(m_data.get(), PG_DIAG_STATEMENT_POSITION)};
     if (p)
       pos = from_string<decltype(pos)>(p);
   }
@@ -497,14 +492,36 @@ char const *pqxx::result::column_name(pqxx::row::size_type number) const &
 
 pqxx::row::size_type pqxx::result::columns() const noexcept
 {
-  auto ptr{const_cast<internal::pq::PGresult *>(m_data.get())};
-  return (ptr == nullptr) ? 0 : row::size_type(PQnfields(ptr));
+  return m_data ? row::size_type(PQnfields(m_data.get())) : 0;
+}
+
+
+int pqxx::result::column_storage(pqxx::row::size_type number) const
+{
+  int const out{PQfsize(m_data.get(), number)};
+  if (out == 0)
+  {
+    auto const sz{this->size()};
+    if ((number < 0) or (number >= sz))
+      throw argument_error{pqxx::internal::concat(
+        "Column number out of range: ", number, " (have 0 - ", sz, ")")};
+    throw failure{pqxx::internal::concat(
+      "Error getting column_storage for column ", number)};
+  }
+  return out;
+}
+
+
+int pqxx::result::column_type_modifier(
+  pqxx::row::size_type number) const noexcept
+{
+  return PQfmod(m_data.get(), number);
 }
 
 
 // const_result_iterator
 
-pqxx::const_result_iterator pqxx::const_result_iterator::operator++(int)
+pqxx::const_result_iterator pqxx::const_result_iterator::operator++(int) &
 {
   const_result_iterator old{*this};
   m_index++;
@@ -512,7 +529,7 @@ pqxx::const_result_iterator pqxx::const_result_iterator::operator++(int)
 }
 
 
-pqxx::const_result_iterator pqxx::const_result_iterator::operator--(int)
+pqxx::const_result_iterator pqxx::const_result_iterator::operator--(int) &
 {
   const_result_iterator old{*this};
   m_index--;
@@ -529,18 +546,18 @@ pqxx::result::const_reverse_iterator::base() const noexcept
 
 
 pqxx::const_reverse_result_iterator
-pqxx::const_reverse_result_iterator::operator++(int)
+pqxx::const_reverse_result_iterator::operator++(int) &
 {
-  const_reverse_result_iterator tmp{*this};
+  const_reverse_result_iterator const tmp{*this};
   iterator_type::operator--();
   return tmp;
 }
 
 
 pqxx::const_reverse_result_iterator
-pqxx::const_reverse_result_iterator::operator--(int)
+pqxx::const_reverse_result_iterator::operator--(int) &
 {
-  const_reverse_result_iterator tmp{*this};
+  const_reverse_result_iterator const tmp{*this};
   iterator_type::operator++();
   return tmp;
 }
