@@ -1,6 +1,6 @@
 /** Various utility functions.
  *
- * Copyright (c) 2000-2023, Jeroen T. Vermeulen.
+ * Copyright (c) 2000-2024, Jeroen T. Vermeulen.
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this
@@ -8,6 +8,8 @@
  */
 #include "pqxx-source.hxx"
 
+#include <array>
+#include <cassert>
 #include <cerrno>
 #include <cmath>
 #include <cstdlib>
@@ -99,42 +101,48 @@ void pqxx::internal::check_unique_unregister(
 
 namespace
 {
-constexpr char hex_digits[] = {'0', '1', '2', '3', '4', '5', '6', '7',
-                               '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
+constexpr std::array<char, 16u> hex_digits{'0', '1', '2', '3', '4', '5',
+                                           '6', '7', '8', '9', 'a', 'b',
+                                           'c', 'd', 'e', 'f'};
 
 
 /// Translate a number (must be between 0 and 16 exclusive) to a hex digit.
 constexpr char hex_digit(int c) noexcept
 {
-  return hex_digits[c];
+  assert(c >= 0 and c < pqxx::internal::ssize(hex_digits));
+  return hex_digits[static_cast<unsigned int>(c)];
 }
 
 
+constexpr int ten{10};
+
+
 /// Translate a hex digit to a nibble.  Return -1 if it's not a valid digit.
 constexpr int nibble(int c) noexcept
 {
   if (c >= '0' and c <= '9')
     PQXX_LIKELY
   return c - '0';
-  else if (c >= 'a' and c <= 'f') return 10 + (c - 'a');
-  else if (c >= 'A' and c <= 'F') return 10 + (c - 'A');
+  else if (c >= 'a' and c <= 'f') return ten + (c - 'a');
+  else if (c >= 'A' and c <= 'F') return ten + (c - 'A');
   else return -1;
 }
 } // namespace
 
 
-void pqxx::internal::esc_bin(
-  std::basic_string_view<std::byte> binary_data, char buffer[]) noexcept
+void pqxx::internal::esc_bin(bytes_view binary_data, char buffer[]) noexcept
 {
   auto here{buffer};
   *here++ = '\\';
   *here++ = 'x';
 
+  constexpr int nibble_bits{4};
+  constexpr int nibble_mask{0x0f};
   for (auto const byte : binary_data)
   {
     auto uc{static_cast<unsigned char>(byte)};
-    *here++ = hex_digit(uc >> 4);
-    *here++ = hex_digit(uc & 0x0f);
+    *here++ = hex_digit(uc >> nibble_bits);
+    *here++ = hex_digit(uc & nibble_mask);
   }
 
   // (No need to increment further.  Facebook's "infer" complains if we do.)
@@ -142,8 +150,7 @@ void pqxx::internal::esc_bin(
 }
 
 
-std::string
-pqxx::internal::esc_bin(std::basic_string_view<std::byte> binary_data)
+std::string pqxx::internal::esc_bin(bytes_view binary_data)
 {
   auto const bytes{size_esc_bin(std::size(binary_data))};
   std::string buf;
@@ -172,10 +179,10 @@ void pqxx::internal::unesc_bin(
   auto out{buffer};
   while (in != end)
   {
-    int hi{nibble(*in++)};
+    int const hi{nibble(*in++)};
     if (hi < 0)
       throw pqxx::failure{"Invalid hex-escaped data."};
-    int lo{nibble(*in++)};
+    int const lo{nibble(*in++)};
     if (lo < 0)
       throw pqxx::failure{"Invalid hex-escaped data."};
     *out++ = static_cast<std::byte>((hi << 4) | lo);
@@ -183,11 +190,10 @@ void pqxx::internal::unesc_bin(
 }
 
 
-std::basic_string<std::byte>
-pqxx::internal::unesc_bin(std::string_view escaped_data)
+pqxx::bytes pqxx::internal::unesc_bin(std::string_view escaped_data)
 {
   auto const bytes{size_unesc_bin(std::size(escaped_data))};
-  std::basic_string<std::byte> buf;
+  pqxx::bytes buf;
   buf.resize(bytes);
   unesc_bin(escaped_data, buf.data());
   return buf;
