@@ -23,7 +23,7 @@ void test_062()
 
   tx.exec0("CREATE TEMP TABLE pqxxbin (binfield bytea)");
 
-  std::string const Esc{tx.esc_raw(std::basic_string<std::byte>{
+  std::string const Esc{tx.esc_raw(bytes{
     reinterpret_cast<std::byte const *>(std::data(TestStr)),
     std::size(TestStr)})};
 
@@ -32,15 +32,15 @@ void test_062()
   result R{tx.exec("SELECT * from pqxxbin")};
   tx.exec0("DELETE FROM pqxxbin");
 
-  auto const B{R.at(0).at(0).as<std::basic_string<std::byte>>()};
+  auto const B{R.at(0).at(0).as<bytes>()};
 
   PQXX_CHECK(not std::empty(B), "Binary string became empty in conversion.");
 
   PQXX_CHECK_EQUAL(
     std::size(B), std::size(TestStr), "Binary string was mangled.");
 
-  std::basic_string<std::byte>::const_iterator c;
-  std::basic_string<std::byte>::size_type i;
+  bytes::const_iterator c;
+  bytes::size_type i;
   for (i = 0, c = std::begin(B); i < std::size(B); ++i, ++c)
   {
     PQXX_CHECK(c != std::end(B), "Premature end to binary string.");
